/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004, Diego Pettenò                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "netloadapplet.h"
#include "../version.h"
#include "settings.h"
#include "interfaces.h"
#include "visualization.h"
#include "knetproc.h"
#include "statgraph.h"
#include "statpopup.h"
#include "statlabel.h"

#include <kaboutapplication.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <qlabel.h>

NetLoadApplet::NetLoadApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
 : KPanelApplet(configFile, type, actions, parent, name), m_labelIn(NULL), m_labelOut(NULL)
{
	kdDebug() << "NetLoadApplet::NetLoadApplet(" << configFile << ", " << type << ", " << actions << ", " << parent << ", " << name << ")" << endl;
	KAboutData aboutData(
		"knetloadapplet",
		I18N_NOOP("KNetLoad Applet"),
		VERSION,
		I18N_NOOP("A Kicker applet of KNetLoad"),
		KAboutData::License_GPL,
		"(C) 2004 Flameeyes",
		I18N_NOOP("See also about data for KNetLoad application"),
		"http://flameeyes.web.ctonet.it/");

	aboutData.addAuthor("Flameeyes", I18N_NOOP("Creator (of applet)"), "flameeyes@users.berlios.de");
	aboutData.addCredit(I18N_NOOP("Thanks obviously to all the creators, maintaines and contributors of KNetLoad Application."));
	m_aboutbox = new KAboutApplication(&aboutData, this, "aboutbox");

	VisualizationConfig *viscfg = new VisualizationConfig(0, "Visualization");
	delete viscfg->kcfg_Split;

	m_cfgdialog = new KConfigDialog(this, "settings", Settings::self());
	m_cfgdialog->addPage(viscfg, i18n("Visualization"), "style");
	InterfacesConfig *ifcfg = new InterfacesConfig(m_popup, "Interfaces");
	const QStringList &ifaces = KNetProc::loadInterfaces(true);
	QStringList::const_iterator enddev = ifaces.end();
	for(QStringList::const_iterator it = ifaces.begin(); it != enddev; it++)
		ifcfg->kcfg_IfName->insertItem(KNetProc::getInterfaceIcon(*it), *it);
	
	m_cfgdialog->addPage(ifcfg, i18n("Interfaces"), "network");
	connect(m_cfgdialog, SIGNAL(settingsChanged()), this, SLOT(reloadSettings()));

	m_readTimer = new QTimer(this);
	connect(m_readTimer, SIGNAL(timeout()), this, SLOT(takeReading()));
	
	m_proc = new KNetProc();
	m_popup = new StatPopup(this, "stat_popup");
	
	m_labelIn = new StatLabel(this, "labelIn");
	connect(m_labelIn, SIGNAL(clicked()), this, SLOT(togglePopup()));
	m_labelOut = new StatLabel(this, "labelOut");
	connect(m_labelOut, SIGNAL(clicked()), this, SLOT(togglePopup()));
	
	reloadSettings();
}

NetLoadApplet::~NetLoadApplet()
{
	delete m_aboutbox;
}

int NetLoadApplet::widthForHeight(int height) const
{
	return (height*2)+2;
}

int NetLoadApplet::heightForWidth(int width) const
{
	return (width-2)/2;
}

void NetLoadApplet::about()
{
	kdDebug() << "NetLoadAPplet::about()" << endl;
	m_aboutbox->show();
}

void NetLoadApplet::preferences()
{
	kdDebug() << "NetLoadApplet::preferences()" << endl;
	if(KConfigDialog::showDialog("settings"))
		return;
	
	m_cfgdialog->show();
}

void NetLoadApplet::reloadSettings()
{
	kdDebug() << "NetLoadApplet::reloadSettings()" << endl;
	QString newDevice = Settings::ifName().stripWhiteSpace();
	if (newDevice.isEmpty())
	{
		KMessageBox::error(0, i18n("The device name cannot be empty."));
		return;
	} else {
		m_proc->setDevice(newDevice);
	}
	
	m_labelIn->graph()->setUpperColor(Settings::color1());
	m_labelIn->graph()->setStyle((StatGraph::Style)Settings::style());
	m_labelIn->graph()->setGridPace( Settings::grid() ? 6 : 0 );
	m_labelIn->graph()->setBgColor( Settings::transparentBackground() ? StatGraph::invalidColor : Settings::colorBackground() );
	m_labelIn->graph()->setLabel( Settings::labelled() ? "IN" : QString::null );
	m_labelIn->graph()->redraw();
		
	m_labelOut->graph()->setUpperColor(Settings::color2());
	m_labelOut->graph()->setStyle((StatGraph::Style)Settings::style());
	m_labelOut->graph()->setGridPace( Settings::grid() ? 6 : 0 );
	m_labelOut->graph()->setBgColor( Settings::transparentBackground() ? StatGraph::invalidColor : Settings::colorBackground() );
	m_labelOut->graph()->setLabel( Settings::labelled() ? "OUT" : QString::null );
	m_labelOut->graph()->redraw();
	
	if (Settings::enabled())
	{
		takeReading();
		m_readTimer->start(Settings::timeout());
	} else {
		m_readTimer->stop();
		m_popup->setText(i18n("Inactive."));
	}
}

void NetLoadApplet::takeReading()
{
	kdDebug() << "NetLoadApplet::takeReading()" << endl;
	m_proc->readLoad();
	
	// Rates in bits per second.
	float m_bitRateIn = m_proc->recentBytesIn() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);
	float m_bitRateOut = m_proc->recentBytesOut() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);

	// Convert to percentages.
	char upperIn = (char) (100.0 * m_bitRateIn / ((float) Settings::scaleIn()));
	char upperOut = (char) (100.0 * m_bitRateOut /
		((float) (Settings::scaleOut() ? Settings::scaleOut() : Settings::scaleIn() )));

	if (upperIn < 0)
		upperIn = 0;
	if (upperOut < 0)
		upperOut = 0;
	if (upperIn > 100)
		upperIn = 100;
	if (upperOut > 100)
		upperOut = 100;
	
	m_labelIn->graph()->addPercentReading(upperIn, 0, Settings::soft());
	m_labelOut->graph()->addPercentReading(upperOut, 0, Settings::soft());
	
	repaint();

	if ( !m_popup->isVisible() ) return;
	
	if ( Settings::speedUnit() == 0 ) // bits per second
	{
		float currBitIn = (int)((m_bitRateIn + 50) / 100) / 10.0;
		float currBitOut = (int)((m_bitRateOut + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Current In: %1 Kb/s, Total In: %2 MB.\n"
			"Current Out: %3 Kb/s, Total Out: %4 MB.")
			.arg(currBitIn).arg(m_proc->totalMbIn())
			.arg(currBitOut).arg(m_proc->totalMbOut()));
	} else { // bytes per second
		float currByteIn = (int)((m_bitRateIn/8 + 50) / 100) / 10.0;
		float currByteOut = (int)((m_bitRateOut/8 + 50) / 100) / 10.0;
		m_popup->setText(i18n(
			"Current In: %1 KB/s, Total In: %2 MB.\n"
			"Current Out: %3 KB/s, Total Out: %4 MB.")
			.arg(currByteIn).arg(m_proc->totalMbIn())
			.arg(currByteOut).arg(m_proc->totalMbOut()));
	}
	m_popup->resizeToText();
}

void NetLoadApplet::togglePopup()
{
	if ( m_popup->isShown() )
		m_popup->hide();
	else
		m_popup->show();
}

void NetLoadApplet::resizeEvent(QResizeEvent *e)
{
	kdDebug() << "NetLoadApplet::resizeEvent(" << e << ")" << endl;
	KPanelApplet::resizeEvent(e);
	
	m_labelIn->setGeometry(0, 0, height(), height());
	m_labelIn->show();
	
	m_labelOut->setGeometry(height()+2, 0, height(), height());
	m_labelOut->show();
	
	update();
	reloadSettings();
}

extern "C"
{
	KPanelApplet* init( QWidget *parent, const QString configFile)
	{
		kdDebug() << "[NetLoadApplet] init(" << parent << ", " << configFile << ")" << endl;
		KGlobal::locale()->insertCatalogue("knetloadapplet");
		return new NetLoadApplet(configFile, KPanelApplet::Normal,
			KPanelApplet::About | KPanelApplet::Preferences,
			parent, "knetloadapplet");
	}
}

#include "netloadapplet.moc"
